<?php

declare(strict_types=1);

namespace Dedi\SyliusContactPlugin\Twig;

use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

class SyliusVersionTesterExtension extends AbstractExtension
{
    // represents 1.7.x
    public const VERSION_DON_T_SHOW_HELP_MANUALLY = [1, 8];

    private array $currentVersion;

    public function __construct(
        string $currentVersion
    ) {
        $version = explode('.', $currentVersion);
        if ($version === false) {
            throw new \LogicException(sprintf(
                'Provided version does not have the correct format: "%s"',
                $currentVersion
            ));
        }

        $this->currentVersion = $version;
    }

    public function getFunctions()
    {
        return [
            new TwigFunction('dedi_sylius_contact_form_test_version', [$this, 'testVersion']),
            new TwigFunction('dedi_sylius_contact_form_test_version_help_message', [$this, 'testVersionHelpMessage']),
        ];
    }

    public function testVersionHelpMessage(): bool
    {
        return $this->testVersion(self::VERSION_DON_T_SHOW_HELP_MANUALLY);
    }

    public function testVersion(array $targetedVersion): bool
    {
        if (count($targetedVersion) > count($this->currentVersion)) {
            throw new \LogicException(sprintf(
                'Target version "%s" has too many digits to be compared against sylius version: "%s".',
                implode('.', $targetedVersion),
                implode('.', $this->currentVersion)
            ));
        }

        foreach ($targetedVersion as $k => $digit) {
            if ($digit > $this->currentVersion[$k]) {
                return false;
            }
        }

        return true;
    }
}
